<?php

namespace Controllers;

// use Teknisa\GER\Utilities\Environment;
// use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
// use Zeedhi\Framework\DTO\Response\Message;

class FrequenciaCicloVooClasseAnual extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName = 'frciclovcla_anual';
    protected $parameterBagColumns = array("DTINICIAL", "DTFINAL", "NRORG", "CDFILIAL", "CDCIA", "CDTRECHO", "NRVOO", "CDCLASSE");


    public function __construct($crudService, $parameterBag,  $environment, $dataSourceManager, $connection) {
        $this->crudService = $crudService;
        $this->parameterBag=$parameterBag;
        $this->environment=$environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->connection = $connection;
    }

    public function save(DTO\Request $request, DTO\Response $response){
    	$this->dataSourceName = 'frciclovcla';
    	$this->parameterBagColumns = array();
		$rows = array();
		$request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
		$dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
		foreach ($request->getDataSet()->getRows() as $row) {
	        $condition = array(
	            array(
	                "value" => $row['CDFILIAL'],
	                "operator" => "=",
	                "columnName" => "CDFILIAL"
	            ),
	            array(
	                "value" => $row['CDCIA'],
	                "operator" => "=",
	                "columnName" => "CDCIA"
	            ),
	            array(
	            	"value" => $row['DTCICLOVCLA'],
	                "operator" => "=",
	                "columnName" => "DTCICLOVCLA"
	            ),
	            array(
	                "value" => $row['NRORG'],
	                "operator" => "=",
	                "columnName" => "NRORG"
	            ),
	            array( 
	            	"value" => $row['NRVOO'],
	            	"operator" => "=",
	            	"columnName" => "NRVOO"
	            ),
	            array(
	            	"value" => $row['CDTRECHO'],
	            	"operator" => "=",
	            	"columnName" => "CDTRECHO"
	            ),
	            array(
	            	"value" => $row['CDCLASSE'],
	            	"operator" => "=",
	            	"columnName" => "CDCLASSE"
	            )
	        );
	        $filterCriteria = new FilterCriteria($this->dataSourceName, $condition);
	        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
	        $newdataSet = $this->dataSourceManager->findBy($newFilterCriteria);
	        $newRows = $newdataSet->getRows();

	        if(!empty($newRows)){
	        	$row['__is_new'] = false;
	        }
	    }
    	parent::save($request, $response);
    }

    public function deleteCiclo(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = 'frciclovcla_cdciclo';
        try {
            $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $this->crudService->delete($response, $dataSet);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 500));
        }
    }
    
    public function importaCicloVooClasse(DTO\Request $request, DTO\Response $response){
    	$dataSource = 'frciclovcla';
    	try{
    		$this->connection->beginTransaction();
	    	$row = $request->getRow();
			$frequenciaNoPeriodo = $this->buscaFrequenciasNoPeriodo($row);
			
			$rowsASeremSalvas = $this->preparaRows($frequenciaNoPeriodo, $row);
			$this->apagaFrequenciaNoPeriodo($row);
			
			$newDataSet = new DataSet($dataSource, $rowsASeremSalvas);
			$newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
			$rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSource);
			$dataSet = new DataSet($dataSource, $rowsToSave);
			$this->dataSourceManager->persist($dataSet);
			$this->connection->commit();
    	} catch(\Exception $e){
    		$this->connection->rollback();
    		$response->setError(new Error($e->getMessage(), 500));
    	}
		
    }
    
    public function exportaCicloVooClasse(DTO\Request $request, DTO\Response $response){
    	$dataSource = 'frciclovcla';
    	try{
    		$this->connection->beginTransaction();
			$row = $request->getRow();
    		for($i = 0; $i < count($row['CHAVE']); $i++){
    			$chave = explode("_",$row['CHAVE'][$i]);
    			$row['NRVOODES'] = $chave[0]; 
    			$row['CDTRECHODES'] = $chave[1]; 
    			$row['CDCLASSEDES'] = $chave[2]; 
				
				$frequenciaNoPeriodo = $this->buscaFrequenciasNoPeriodo($row);
				$rowsASeremSalvas = $this->preparaRows($frequenciaNoPeriodo, $row);
				
				$this->apagaFrequenciaNoPeriodo($row);
				$newDataSet = new DataSet($dataSource, $rowsASeremSalvas);
				$newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
				$rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSource);
				$dataSet = new DataSet($dataSource, $rowsToSave);
				$this->dataSourceManager->persist($dataSet);
    		}
    		$this->connection->commit();
		} catch(\Exception $e){
			$this->connection->rollback();
    		$response->setError(new Error($e->getMessage(), 500));
    	}
    }
    
    private function buscaFrequenciasNoPeriodo($row){
    	
    	$query = 'SELECT CDFILIAL,
					     CDCIA,
					     CDCICLO,
					     NRVOO,
					     CDTRECHO,
					     CDCLASSE,
					     DTCICLOVCLA,
					     NRORG
					FROM FRCICLOVCLA
				   WHERE CDFILIAL = :CDFILIAL
					 AND CDCIA = :CDCIA
					 AND NRVOO = :NRVOO
					 AND CDTRECHO = :CDTRECHO
					 AND CDCLASSE = :CDCLASSE
					 AND DTCICLOVCLA BETWEEN :DTINICIAL AND :DTFINAL
					 AND NRORG = :NRORG';
		$stmt = $this->connection->prepare($query);
		$params = array(
			'CDFILIAL'  => $row['CDFILIAL'],
			'CDCIA'     => $row['CDCIA'],
			'NRVOO'     => $row['NRVOO'],
			'CDTRECHO'  => $row['CDTRECHO'],
			'CDCLASSE'  => $row['CDCLASSE'],
			'DTINICIAL' => $row['DTINIVGMA'],
			'DTFINAL'   => $row['DTFIMVGMA'],
			'NRORG' => $this->environment->getNrOrgTrab()
		);
		
		$stmt->execute($params);
		
		$result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
		
		return $result;
    	
    }
    
    private function apagaFrequenciaNoPeriodo($row){
    	
    	$query = 'DELETE FROM FRCICLOVCLA
                   WHERE CDFILIAL = :CDFILIAL
                     AND CDCIA = :CDCIA
                     AND CDTRECHO = :CDTRECHO
                     AND NRVOO = :NRVOO
                     AND CDCLASSE = :CDCLASSE
                     AND DTCICLOVCLA BETWEEN :DTINICIAL AND :DTFINAL
                     AND NRORG = :NRORG';
                     
		$stmt = $this->connection->prepare($query);
		
		$params = array(
			'CDFILIAL'  => $row['CDFILIAL'],
			'CDCIA'     => $row['CDCIA'],
			'NRVOO'     => $row['NRVOODES'],
			'CDTRECHO'  => $row['CDTRECHODES'],
			'CDCLASSE'  => $row['CDCLASSEDES'],
			'DTINICIAL' => $row['DTINIVGMA'],
			'DTFINAL'   => $row['DTFIMVGMA'],
			'NRORG' => $this->environment->getNrOrgTrab()
		);
		
		$stmt->execute($params);
		
    }
    
	private function preparaRows($rowsImportadas, $row){
    	    	
		foreach ($rowsImportadas as &$item) {
			$item['CDTRECHO'] = $row['CDTRECHODES'];
			$item['NRVOO'] = $row['NRVOODES'];
			$item['CDCLASSE'] = $row['CDCLASSEDES'];
			$item['CDCIA'] = $row['CDCIA'];
			$item['changed'] = false;
			$item['__is_new'] = true;
		}
		
		return $rowsImportadas;
	
    }

}