<?php

namespace Services;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class WorkflowService {
	protected $crudService;
	protected $novoCodigo;
	protected $connection;
	protected $menuDesignService;

	public function __construct($crudService, $novoCodigo, $connection, $menuDesignService){
		$this->crudService = $crudService;
		$this->novoCodigo = $novoCodigo;
		$this->connection = $connection;
		$this->menuDesignService = $menuDesignService;
	}

	public function createRowToWorkFlowProcess(array $dataToSave, array $primaryKeys, $dsmotivowflow, $nmitemwflow, $nmtabelawflow, $idtipoperwflow, $nritemprocaprova, $nrprocessoaprova){
		$infoAtual;
		$nmcampos;
		if(isset($dataToSave["__leftActions"]))
			unset($dataToSave["__leftActions"]);
		if(isset($dataToSave["__rightActions"]))
			unset($dataToSave["__rightActions"]);
		if(isset($dataToSave["__columns_to_return"]))
			unset($dataToSave["__columns_to_return"]);
		if(isset($dataToSave['DSMONTASUBSEMAMHTML'])){
			$infoAtual = $dataToSave['IDANALIZEATIV'] . ';' . $dataToSave['DSMONTASUBSEMAMHTML'];
		}
		else {
			$infoAtual = implode(';',$dataToSave);
		}
		$row = array(
			'__is_new' => true,
			'NMCAMPOWFLOW' => implode(';',array_keys($dataToSave)),
			'DSINFORATUAL' => $infoAtual,
			'DSPRIMARYKEY' => $this->rowToStr($primaryKeys),
			'NMITEMWFLOW' => $nmitemwflow,
			'DSMOTIVOWFLOW' => $dsmotivowflow,
			'NMTABELAWFLOW' => $nmtabelawflow,
			'IDTIPOPERWFLOW' => strval($idtipoperwflow), // '1' - Update, '2' - Intert, '3' - Delete
			'NRITEMPROCAPROVA' => intval($nritemprocaprova),
			'NRPROCESSOAPROVA' => intval($nrprocessoaprova)
		);
		return $row;
	}

	public function createWorkFlowProcess(array $primaryKeys, array $saveData, $rowWorkflow, $updateCheckColumn = false, $checkColumn = null, $columnValue = 'P'){
		// if($this->useWorkflow($rowWorkflow)){
			$processOperators = $this->checkOperatorWorkFlow($rowWorkflow);
			$nrworkflow = $this->executeWorkflow($rowWorkflow);
			$rowWorkflow['NRWORKFLOW'] = $nrworkflow;
			$this->newDadosWflow($primaryKeys, $saveData, $rowWorkflow);
			$this->newOperworkflow($rowWorkflow, $processOperators);

			// if ($updateCheckColumn) {
			// 	$this->updateCheckColumn($checkColumn, $columnValue, $primaryKeys, $rowWorkflow['NMTABELAWFLOW']);
			// }
		// } else {
		// 	$this->executeOperation($operationId, $pimaryKeys, $saveData);
		// }
		return $nrworkflow;
	}

	private function updateCheckColumn($checkColumn, $columnValue, $primaryKeys, $tableName) {
		$tableName = strtolower($tableName);
		$conditions = array();
		foreach ($primaryKeys as $primaryKey => $value) {
			$condition = array(
				'columnName' => $primaryKey,
				'operator' => '=',
				'value' => $value
			);
			$conditions[] = $condition;
		}
		$row = $this->crudService->getFilteredDataSet($tableName, $conditions)->getRows()[0];
		$row[$checkColumn] = $columnValue;
		$row['changed'] = true;
		$request = $this->mountRequest(array($row), $tableName);
		$this->save($request, $tableName);
	}

	private function newOperworkflow(array $rowWorkflow, array $processOperators){
		$rowsToSave = array();
		foreach($processOperators as $operator){
			$aditionalData = array(
				'CDOPERADOR' => $operator['CDOPERADOR'],
				'IDSITOPERWFLOW' => 'P',
				'DTLIMITEWFLOW' => $this->incrementDate($operator['NRDIASOPERAPRO'])
			);
			$rowToSave = array_merge($aditionalData, $rowWorkflow);
			$rowsToSave[] = $rowToSave;
		}
		$requestSave = $this->mountRequest($rowsToSave, 'operworkflow');
		$this->save($requestSave, 'operworkflow');
	}

	private function incrementDate($daysIncrement){
		$date = new \DateTime();
		$daysIncrement = strval($daysIncrement);
		$date->add(new \DateInterval('P'.$daysIncrement.'D'));
		$date->setTime(23, 59,59);
		return $date->format('d/m/Y H:i:s');
	}

	private function executeWorkflow(array $rowWF){
		$nrworkflow  = $this->novoCodigo->getCodeByTableName('WORKFLOW', $this->crudService->getEnvironment()->getNrOrg(), 1, 10, true);
		$rowWF['NRWORKFLOW'] = $nrworkflow;
		$this->newWorkflow($rowWF);
		return $nrworkflow;
	}

	private function newDadosWflow($primaryKeys, $saveData, $rowWF){
		$oldData = $this->getOldData($rowWF, $primaryKeys, $saveData);
		$aditionalData = array(
			'DSINFORANTERIOR' => $oldData
		);
		$rowSave = array_merge($rowWF, $aditionalData);
		$request = $this->mountRequest(array($rowSave), 'dadoswflow');
		$this->save($request, 'dadoswflow');
	}

	private function newWorkflow(array $rowWF){
		$aditionalData = array(
			'CDOPERADOR' => $this->crudService->getEnvironment()->getCdOperador(),
			'IDSITWORKFLOW' => '1'
		);

		$rowSave = array_merge($rowWF, $aditionalData);
		$request = $this->mountRequest(array($rowSave), 'workflow');
		$this->save($request, 'workflow');
	}

	public function rowToStr($row){
		$dataInStr = '';
		foreach($row as $index => $key){
			$dataInStr = $dataInStr.$index.'='.$key.';';
		}
		return $dataInStr;
	}

	private function getOldData($rowWF, $primaryKeys, $saveData){
		$conditions = array();
		$dataSourceName = strtolower($rowWF['NMTABELAWFLOW']);
		foreach ($primaryKeys as $primaryKey => $value) {
			$condition = array(
				'columnName' => $primaryKey,
				'operator' => '=',
				'value' => $value
			);
			$conditions[] = $condition;
		}
		$rows = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		if (!empty($rows)) {
			$row = $rows[0];
			$oldRow = array();
			foreach ($saveData as $index => $value) {
				$oldRow[$index] = (isset($row[$index])) ? $row[$index] : null;
			}
			$dataReturn = implode(';', $oldRow);
			if (empty($dataReturn)) {
				return 'N/A';
			}
			return implode(';', $oldRow);
		}

		return 'N/A';
	}

	private function useWorkflow($dataOperation){
		$conditions = $this->getConditionsItemprocaprova($dataOperation);
		$operationWorkFlow = $this->crudService->getFilteredDataSet('itemprocaprova', $conditions)->getRows()[0];
		return ($operationWorkFlow['IDATIVO'] == 'S');
	}

	private function checkOperatorWorkFlow($dataOperation){
		$conditions = $this->getConditionsItemprocaprova($dataOperation);

		$operatorProcess = $this->crudService->getFilteredDataSet('operitempaprova', $conditions)->getRows();
		if(empty($operatorProcess)){
			throw new \Exception("No have operator to this WorkFlow operation");
		}
		return $operatorProcess;
	}

	private function checkPrimaryKeys($operationId, array $pimaryKeys){
		if(empty($pimaryKeys)){
			throw new \Exception("Primary keys can't be null");
		}
		$localPks = $this->operationMapping[$operationId]['PRIMARY_KEYS'];
		$userPks = array_keys($primaryKeys);

		$diffPks = array_diff($localPks, $userPks);
		if(!empty($diffPks)){
			throw new \Exception("Wrong primary keys: ". implode(', ',$diffPks));
		}
	}

	private function getConditionsItemprocaprova($dataOperation){
		$conditions = array(
			array(
				'columnName' => 'NRPROCESSOAPROVA',
				'operator' => '=',
				'value' => $dataOperation['NRPROCESSOAPROVA']
			),
			array(
				'columnName' => 'NRITEMPROCAPROVA',
				'operator' => '=',
				'value' => $dataOperation['NRITEMPROCAPROVA']
			)
		);
		return $conditions;
	}

	private function mountRequest($rows, $dataSourceName){
		$dataSet = new DataSet($dataSourceName, $rows);
		$request = new DTO\Request\DataSet($dataSet, '','','');
		return $request;
	}

	public function executeOperation($rowOperation, array $primaryKeys, array $saveData){
		$dataError = array();
		if($rowOperation['NRITEMPROCAPROVA'] == 24 && $rowOperation['NRPROCESSOAPROVA'] == 4){
			$this->executeProcedeureManutencaoMenuGrid($primaryKeys);
		} else if ($rowOperation['NRITEMPROCAPROVA'] == 1 && $rowOperation['NRPROCESSOAPROVA'] == 4) {
			$this->executaAprocaoCusteioMenuGrid($rowOperation, $primaryKeys, $saveData);
		} else if ($rowOperation['NRITEMPROCAPROVA'] == 25 && $rowOperation['NRPROCESSOAPROVA'] == 4) {
			$dataError = $this->changeVigenciaMenuGrid($saveData, $primaryKeys);
		} else {
			$rowSave = array_merge($primaryKeys, $saveData);
			$rowSave['__is_new'] = ($rowOperation['IDTIPOPERWFLOW'] == '2');
			$rowSave['changed'] = true;
			$dataSourceName = strtolower($rowOperation['NMTABELAWFLOW']);
			$requestSave = $this->mountRequest(array($rowSave), $dataSourceName);
			if($rowOperation['IDTIPOPERWFLOW'] == '3'){
				$this->delete($requestSave, $dataSourceName);
			} else {
				$this->save($requestSave, $dataSourceName);
			}
		}
		return $dataError;
	}

	private function save($request, $dataSourceName){
		$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
		$this->crudService->save($dataSourceName, $rows);
	}

	private function delete($request, $dataSourceName){
		$rows = $this->crudService->getRowsToDelete($request, $dataSourceName);
		$dataSet = new DataSet($dataSourceName, (array)$rows);
		$this->crudService->delete(new DTO\Response(),$dataSet);
	}

	public function canExecuteWorkflowOperation($conditionsOperworkflow){
		$processOperators = $this->crudService->getFilteredDataSet('coc00004_operworkflow_operitempaprova', $conditionsOperworkflow)->getRows();
		if(empty($processOperators)){
			return true;
		}
		return false;
	}

	public function getDataToExecuteOperation($nrworkflow){
		$conditions = array(
			array(
				'columnName' => 'NRWORKFLOW',
				'operator' => '=',
				'value' => $nrworkflow
			),
			array(
				'columnName' => 'NRORG',
				'operator' => '=',
				'value' => $this->crudService->getEnvironment()->getNrOrg()
			)
		);

		$data = $this->crudService->getFilteredDataSet('dadoswflow', $conditions)->getRows()[0];
		
		$nmcampos = explode(';',$data['NMCAMPOWFLOW']);
		$informacoesCampos;
		if(in_array('DSMONTASUBSEMAMHTML', $nmcampos)){
			$informacoesCampos[] = strstr($data['DSINFORATUAL'], ';', true);
			$dsHtml = strstr($data['DSINFORATUAL'], ';');
			$informacoesCampos[] = substr_replace($dsHtml, '', 0, 1);
		} else {
			$informacoesCampos = explode(';',$data['DSINFORATUAL']);
		}
		$dsPrimaryKey = explode(';',$data['DSPRIMARYKEY']);
		$rowDataSave = array();
		foreach($nmcampos as $index => $nmcampo){
			$rowDataSave[$nmcampo] = $informacoesCampos[$index];
		}
		array_pop($dsPrimaryKey);
		$primaryKeys = array();
		foreach($dsPrimaryKey as $datapk){
			$primaryKey = explode('=', $datapk);
			if (sizeof($primaryKey) > 1) {
				$primaryKeys[$primaryKey[0]] = $primaryKey[1];
			}
		}

		$dataWorkflow = $this->crudService->getFilteredDataSet('workflow', $conditions)->getRows()[0];
		$rowOperation = array(
			'NMTABELAWFLOW' => $data['NMTABELAWFLOW'],
			'IDTIPOPERWFLOW' => $dataWorkflow['IDTIPOPERWFLOW'],
			'NRITEMPROCAPROVA' => $dataWorkflow['NRITEMPROCAPROVA'],
			'NRPROCESSOAPROVA' => $dataWorkflow['NRPROCESSOAPROVA']
		);
		return array('dataSave' => $rowDataSave, 'primaryKeys' => $primaryKeys, 'rowOperation' => $rowOperation);
	}

	public function changeWorkFlowStatus($row, $idsitworkflow){
		$rowSave = array(
			'__is_new' => false,
			'NRWORKFLOW' => $row['NRWORKFLOW'],
			'IDSITWORKFLOW' => $idsitworkflow
		);
		$requestSave = $this->mountRequest(array($rowSave), 'workflow');
		$this->save($requestSave, 'workflow');
	}

	public function changeOperWorkflowStatus($row){
		$dateTime = new \DateTime();
		$rowSave = array(
			'__is_new' => false,
			'IDSITOPERWFLOW' =>$row['IDSITOPERWFLOW'],
			'DTACAOWFLOW' =>  $dateTime->format('d/m/Y H:i:s'),
			'DSMOTIVOACAO' => $row['DSMOTIVOACAO'],
			'CDOPERADOR' => $this->crudService->getEnvironment()->getCdOperador(),
			'NRWORKFLOW' => $row['NRWORKFLOW']
		);

		$requestSave = $this->mountRequest(array($rowSave), 'operworkflow');
		$this->save($requestSave, 'operworkflow');
	}

	public function deleteByOperation(array $primaryKeys, $tableName){
		$rowDelete = $primaryKeys;
		$rowDelete['__is_new'] = false;
		$dataSourceName = strtolower($tableName);
		$requestDelete = $this->mountRequest(array($rowDelete), $dataSourceName);
		$this->delete($requestDelete, $dataSourceName);
	}

	private function executeProcedeureManutencaoMenuGrid($row){
		$environment = $this->crudService->getEnvironment();

		$procedure = new StoredProcedure($this->connection, 'EFETIVA_SUBGRU_DUPLICADO');
		$procedure->addParam(new Param('CDFILIAL',         Param::PARAM_INPUT, $row['CDFILIAL']));
		$procedure->addParam(new Param('CDCIA',            Param::PARAM_INPUT, $row['CDCIA']));
		$procedure->addParam(new Param('CDMANCIA',         Param::PARAM_INPUT, $row['CDMANCIA']));
		$procedure->addParam(new Param('NRSERVICOCIAFILI', Param::PARAM_INPUT, $row['NRSERVICOCIAFILI']));
		$procedure->addParam(new Param('CDCLASSE',         Param::PARAM_INPUT, $row['CDCLASSE']));
		$procedure->addParam(new Param('CDSUBGRU',         Param::PARAM_INPUT, $row['CDSUBGRU']));
		$procedure->addParam(new Param('NRSEQSUBGRUSEMAM', Param::PARAM_INPUT, $row['NRSEQSUBGRUSEMAM']));
		$procedure->addParam(new Param('CDOPERADOR',       Param::PARAM_INPUT, $environment->getCdOperador()));
		$procedure->addParam(new Param('NRORG',            Param::PARAM_INPUT, $environment->getNrOrg()));

		$procedure->execute();
	}

	public function executaAprocaoCusteioMenuGrid($rowOperation, $primaryKeys, $saveData) {
		$selectSubgrupos = "SELECT * FROM SUBGRUSEMAM WHERE CDCIA = :CDCIA AND CDFILIAL  = :CDFILIAL AND CDMANCIA = :CDMANCIA AND NRORG = :NRORG";
		$paramsSubgrupo = array(
			'NRORG' => $this->crudService->getEnvironment()->getNrOrg(),
			'CDCIA' => $primaryKeys['CDCIA'],
			'CDFILIAL' => $primaryKeys['CDFILIAL'],
			'CDMANCIA' => $primaryKeys['CDMANCIA']
		);
		$subgrupos = $this->connection->fetchAll($selectSubgrupos, $paramsSubgrupo);
		foreach ($subgrupos as &$subgrupo) {
			foreach ($saveData as $key => $value) {
				$subgrupo[$key] = $value;
			}
			$subgrupo['changed'] = true;
			$subgrupo['__is_new'] = false;
		}
		$dataSourceName = strtolower($rowOperation['NMTABELAWFLOW']);
		$request = $this->mountRequest($subgrupos, $dataSourceName);
		$this->save($request, $dataSourceName);

		$environment = $this->crudService->getEnvironment();
		$nrorg = $environment->getNrOrg();
		$cdoperador = $environment->getCdOperador();

		$sequencialReceita = 'SELECT COUNT(*) AS CONTADOR FROM RECSUBSEMAM WHERE CDFILIAL = :CDFILIAL AND CDCIA = :CDCIA AND CDMANCIA = :CDMANCIA AND NRORG = :NRORG';
		$params = array(
            	"CDFILIAL" =>	$primaryKeys["CDFILIAL"],
            	"CDCIA" =>		$primaryKeys["CDCIA"],
            	"CDMANCIA" =>	$primaryKeys["CDMANCIA"],
            	"NRORG" =>		$environment->getNrOrg()
		);
		$result = $this->connection->fetchAll($sequencialReceita, $params);
		$contador = intval($result[0]['CONTADOR']);
		if($contador > 0) {
			$nrsequencialReceita = $this->novoCodigo->getCodeByTableName('HISTCUSTORECMAM', $nrorg, $contador, 10, true);
			$insertSelectReceita = "INSERT INTO HISTCUSTORECMAM( CDFILIAL, CDCIA, CDMANCIA, NRSERVICOCIAFILI, CDCLASSE, CDSUBGRU, NRSEQSUBGRUSEMAM, CDCICLO, NRSEQHISTCUSTORECMAM, CDRECEITA, VRCUSRESUBSEMAM, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDSITUACAO) SELECT CDFILIAL, CDCIA, CDMANCIA, NRSERVICOCIAFILI, CDCLASSE, CDSUBGRU, NRSEQSUBGRUSEMAM, CDCICLO, (ROWNUM-1+:NRSEQUENCIAL), CDRECEITA, VRCUSRESUBSEMAM, :NRORG, IDATIVO, SYSDATE, SYSDATE, :NRORG, :CDOPERADOR, :NRORG, :CDOPERADOR, 'A' AS IDSITUACAO FROM RECSUBSEMAM WHERE CDFILIAL = :CDFILIAL AND CDCIA = :CDCIA AND CDMANCIA = :CDMANCIA AND NRORG = :NRORG";
			$stmt = $this->connection->prepare($insertSelectReceita);
			$stmt->execute(
				array(
	            	"CDFILIAL" =>	$primaryKeys["CDFILIAL"],
	            	"CDCIA" =>		$primaryKeys["CDCIA"],
	            	"CDMANCIA" =>	$primaryKeys["CDMANCIA"],
	            	"CDOPERADOR" => $cdoperador,
	            	"NRORG" =>		$nrorg,
	            	"NRSEQUENCIAL" => $nrsequencialReceita
	            )
			);
		}

		$sequencialProduto = 'SELECT COUNT(*) AS CONTADOR FROM PRODSUBSEMAM WHERE CDFILIAL = :CDFILIAL AND CDCIA = :CDCIA AND CDMANCIA = :CDMANCIA AND NRORG = :NRORG';
		$params = array(
            	"CDFILIAL" =>	$primaryKeys["CDFILIAL"],
            	"CDCIA" =>		$primaryKeys["CDCIA"],
            	"CDMANCIA" =>	$primaryKeys["CDMANCIA"],
            	"NRORG" =>		$environment->getNrOrg()
		);
		$result = $this->connection->fetchAll($sequencialProduto,$params);
		$contador = intval($result[0]['CONTADOR']);
		if($contador > 0) {
			$nrsequencialProduto = $this->novoCodigo->getCodeByTableName('HISTCUSTOPRODMAM', $nrorg, $contador, 10, true);

			$insertSelectProduto = "INSERT INTO HISTCUSTOPRODMAM( CDFILIAL, CDCIA, CDMANCIA, NRSERVICOCIAFILI, CDCLASSE, CDSUBGRU, NRSEQSUBGRUSEMAM, CDCICLO, NRSEQHISTCUSTOPRODMAM, CDPRODUTO, VRCUSPRSUBSEMAM, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDSITUACAO) SELECT CDFILIAL, CDCIA, CDMANCIA, NRSERVICOCIAFILI, CDCLASSE, CDSUBGRU, NRSEQSUBGRUSEMAM, CDCICLO, (ROWNUM-1+:NRSEQUENCIAL), CDPRODUTO, VRCUSPRSUBSEMAM, :NRORG, IDATIVO, SYSDATE, SYSDATE, :NRORG, :CDOPERADOR, :NRORG, :CDOPERADOR, 'A' AS IDSITUACAO FROM PRODSUBSEMAM WHERE CDFILIAL = :CDFILIAL AND CDCIA = :CDCIA AND CDMANCIA = :CDMANCIA AND NRORG = :NRORG ";
			$stmt = $this->connection->prepare($insertSelectProduto);
			$stmt->execute(
				array(
	            	"CDFILIAL" =>	$primaryKeys["CDFILIAL"],
	            	"CDCIA" =>		$primaryKeys["CDCIA"],
	            	"CDMANCIA" =>	$primaryKeys["CDMANCIA"],
	            	"CDOPERADOR" => $cdoperador,
	            	"NRORG" =>		$nrorg,
	            	"NRSEQUENCIAL" => $nrsequencialProduto
	            )
			);
		}
	}

	private function changeVigenciaMenuGrid($dataSave, $primaryKeys){
		$this->menuDesignService->extendDateSubgrupo(array_merge($dataSave, $primaryKeys));
		$conditionsMenuGrid = array();
		foreach($primaryKeys as $key => $value){
			array_push($conditionsMenuGrid,
				array(
					'columnName' => $key,
					'operator' => '=',
					'value' => $value
				)
			);
		}
		$manualciam = $this->crudService->getFilteredDataSet('manualciam', $conditionsMenuGrid)->getRows()[0];
		foreach ($dataSave as $key => $value) {
			$manualciam[$key] = $value;
		}
		$requestSave = $this->mountRequest(array($manualciam), 'manualciam');
		$this->save($requestSave, 'manualciam');
		$subgrupoSobreposto = $this->menuDesignService->verifyOverlapSubgrupo($primaryKeys);
		$dataError = array();
		$subgrupos = array();
		if(!empty($subgrupoSobreposto))
			$dataError = $this->menuDesignService->manipulateDataError($subgrupoSobreposto, $primaryKeys);
		return $dataError;
	}

	public function changeStatusVersao($row){
		$dataOperation = $this->getDataToExecuteOperation($row['NRWORKFLOW']);
		$rowSave = $dataOperation['primaryKeys'];
		$rowSave['__is_new'] = false;
		$rowSave['changed'] = true;
		$rowSave['IDPENDENTE'] = 'N';
		$rowSave['IDSITUACAO'] = 'E';
		$dataSourceName = strtolower($dataOperation['rowOperation']['NMTABELAWFLOW']);
		$requestSave = $this->mountRequest(array($rowSave), $dataSourceName);
		$this->save($requestSave, $dataSourceName);
	}
}